// -*- C++ -*-
/*!
 * @file  RobotSensorDataManager.h
 * @brief sensor data management
 * @date  $Date$
 *
 * @author East Japan Accounting Center Co.,Ltd.
 *
 * (c)2017 The University of Aizu
 * This software is released under the New BSD License.
 *
 * $Id$
 */

#ifndef ROBOTSENSORDATAMANAGER_H
#define ROBOTSENSORDATAMANAGER_H

#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/idl/ExtendedDataTypesSkel.h>
#include <rtm/idl/InterfaceDataTypesSkel.h>

// Service implementation headers
// <rtc-template block="service_impl_h">
#include "RobotSensorDataManagerSVC_impl.h"
#include "VersatileOverheadSystem.h"
#include "json.h"
#include "writer.h"
#include <chrono>

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="port_stub_h">
// </rtc-template>

using namespace RTC;

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>

//************************************************
struct SensorData_st{
	double_type	dGyro_Roll;
	double_type	dGyro_Pitch;
	double_type	dGyro_Yaw;
	double_type	dSpeed_XAxis;
	double_type	dSpeed_YAxis;
	double_type	dSpeed_Yaw;
	double_type	dSpeed;
	double_type	dTemp;
	double_type	dCompass;
	double_type	dWindPress;
};

struct RobotSensorData_st{
	std::string strRobotName;
	SensorData_st stSensorData;
};


enum SensorRobotID{
	SENSOR_ID_SPIDER = 0,
	SENSOR_ID_DRONE_1,
	SENSOR_ID_DRONE_2,
	SENSOR_ID_DRONE_3,
	SENSOR_ID_GORONE_1,
	SENSOR_ID_GORONE_2,
	SENSOR_ID_GORONE_3,
	SENSOR_ID_GORONE_4
};

//************************************************

/*!
 * @class RobotSensorDataManager
 * @brief sensor data management
 *
 * 別紙"機能仕様書_ロボットセンサデータ管理RTC.docx"参照
 *
 * 別紙"機能仕様書_ロボットセンサデータ管理RTC.docx"参照
 *
 * 別紙"機能仕様書_ロボットセンサデータ管理RTC.docx"参照
 *
 * 別紙"機能仕様書_ロボットセンサデータ管理RTC.docx"参照
 *
 */
class RobotSensorDataManager
  : public RTC::DataFlowComponentBase
{
 public:
  /*!
   * @brief constructor
   * @param manager Maneger Object
   */
  RobotSensorDataManager(RTC::Manager* manager);

  /*!
   * @brief destructor
   */
  ~RobotSensorDataManager();

  // <rtc-template block="public_attribute">
  
  // </rtc-template>

  // <rtc-template block="public_operation">
  
  // </rtc-template>

  /***
   * ロボットセンサデータ管理RTCの初期化処理
   *
   * The initialize action (on CREATED->ALIVE transition)
   * formaer rtc_init_entry() 
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post -
   * 
   */
   virtual RTC::ReturnCode_t onInitialize();

  /***
   * ロボットセンサデータ管理RTCの終了処理
   *
   * The finalize action (on ALIVE->END transition)
   * formaer rtc_exiting_entry()
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post -
   * 
   */
   virtual RTC::ReturnCode_t onFinalize();

  /***
   *
   * The startup action when ExecutionContext startup
   * former rtc_starting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  /***
   *
   * The shutdown action when ExecutionContext stop
   * former rtc_stopping_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  /***
   * ロボットセンサデータ管理RTCの開始処理
   *
   * The activated action (Active state entry action)
   * former rtc_active_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post -
   * 
   */
   virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  /***
   * ロボットセンサデータ管理RTCの停止処理
   *
   * The deactivated action (Active state exit action)
   * former rtc_active_exit()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post -
   * 
   */
   virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  /***
   * ロボットセンサデータ管理RTCの周期処理
   *
   * The execution action that is invoked periodically
   * former rtc_active_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post -
   * 
   */
   virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  /***
   * ロボットセンサデータ管理RTCのエラー遷移処理
   *
   * The aborting action when main logic error occurred.
   * former rtc_aborting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post -
   * 
   */
   virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  /***
   *
   * The error action in ERROR state
   * former rtc_error_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  /***
   *
   * The reset action that is invoked resetting
   * This is same but different the former rtc_init_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  /***
   *
   * The state update action that is invoked after onExecute() action
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  /***
   *
   * The action that is invoked when execution context's rate is changed
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // <rtc-template block="protected_attribute">
  
  // </rtc-template>

  // <rtc-template block="protected_operation">
  
  // </rtc-template>

  // Configuration variable declaration
  // <rtc-template block="config_declare">
  /*!
   * GPSデータ出力周期を調整する
   * - Name: Output_Cycle Output_Cycle
   * - DefaultValue: 0
   * - Unit: ms
   * - Range: 0<=x<=10000
   * - Constraint: -
   */
  long int m_Output_Cycle;

  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  RTC::TimedStringSeq m_inSpiderSensorData;
  /*!
   * 接続台数分のスパイダーのセンサデータをJson形式で取得するデータ
   * ポート
   * - Type: RTC::TimedStringSeq
   * - Number: 1
   * - Frequency: ロボットに依存
   * - Operation Cycle: ロボットに依存
   */
  InPort<RTC::TimedStringSeq> m_inSpiderSensorDataIn;
  RTC::TimedStringSeq m_inDroneSensorData;
  /*!
   * 接続台数分のドローンのセンサデータをJson形式で取得するデータポ
   * ート
   * - Type: RTC::TimedStringSeq
   * - Number: 3
   * - Frequency: ロボットに依存
   * - Operation Cycle: ロボットに依存
   */
  InPort<RTC::TimedStringSeq> m_inDroneSensorDataIn;
  RTC::TimedStringSeq m_inGoroneSensorData;
  /*!
   * 接続台数分のゴローンのセンサデータをJson形式で取得するデータポ
   * ート
   * - Type: RTC::TimedStringSeq
   * - Number: 4
   * - Frequency: ロボットに依存
   * - Operation Cycle: ロボットに依存
   */
  InPort<RTC::TimedStringSeq> m_inGoroneSensorDataIn;
  
  // </rtc-template>


  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  RTC::TimedString m_outRobotSensorData;
  /*!
   * 対象ロボットのセンサデータをJson形式（汎用化したフォーマット）
   * で提供するデータポート
   * - Type: RTC::TimedString
   * - Number: 1
   * - Frequency: 500ms周期
   * - Operation Cycle: 500ms周期
   */
  OutPort<RTC::TimedString> m_outRobotSensorDataOut;
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  /*!
   * センサデータ出力制御関連のインターフェース
   * Interface: センサデータ出力対象ロボット選択などのインターフェ
   *            ースを提供する
   */
  RTC::CorbaPort m_SensorDataManagerReqServPort;
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  /*!
   * センサデータ出力対象ロボット選択要求を行う
   * - Argument:      別紙"機能仕様書_ロボットセンサデータ管理RTC.
   *                  docx"参照
   * - Return Value:  別紙"機能仕様書_ロボットセンサデータ管理RTC.
   *                  docx"参照
   * - Exception:     なし
   * - PreCondition:  -
   * - PostCondition: -
   */
  SensorDataOutputSelectSVC_impl m_SensorDataOutputSelectProv;
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  
  // </rtc-template>

 private:
  // <rtc-template block="private_attribute">
  
  // </rtc-template>

  // <rtc-template block="private_operation">
  
  // </rtc-template>

	void nvdReadSpiderSensorData(void);
	void nvdReadDroneSensorData(void);
	void nvdReadGoroneSensorData(void);
	uhword_type uhwRobotSensorDataOutput(void);
	void nvdRobotSensorDataReqSet(void);
	void nvdSetSelectRobotData(uhword_type,RobotSensorData_st);
	void nvdJsonDataInit(void);
	void nvdRobotSensorDataInit(void);

	RobotSensorData_st m_stSpiderSensorData;	//スパイダーセンサデータを格納
	RobotSensorData_st m_stDrone1SensorData;	//ドローン_1センサデータを格納
	RobotSensorData_st m_stDrone2SensorData;	//ドローン_2センサデータを格納
	RobotSensorData_st m_stDrone3SensorData;	//ドローン_3センサデータを格納
	RobotSensorData_st m_stGorone1SensorData;	//ゴローン_1センサデータを格納
	RobotSensorData_st m_stGorone2SensorData;	//ゴローン_2センサデータを格納
	RobotSensorData_st m_stGorone3SensorData;	//ゴローン_3センサデータを格納
	RobotSensorData_st m_stGorone4SensorData;	//ゴローン_4センサデータを格納

	RobotSensorData_st *RobotSensorData_p[SENSOR_SUPPORT_ROBOT_MAX];

	Json::Value m_JsonData;						//Jsonデータ格納
	uword_type m_uwCurrentOutputSel;			//現在のロボット選択状態

	std::chrono::system_clock::time_point  m_LastTime;	//前回のセンサデータ出力時間を保持

};


extern "C"
{
  DLL_EXPORT void RobotSensorDataManagerInit(RTC::Manager* manager);
};

#endif // ROBOTSENSORDATAMANAGER_H
